#pragma once
#include <FL/Fl_Gl_Window.h>
#include <Fl/Fl.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <Fl/Fl_Double_Window.h>
#include <Fl/Fl_Button.h>

#include "SQRT3.h"
#include "Loops.h"

extern int SHADING_STATE;


static GLfloat g_lightAmbient[]  =	{ 0.1f, 0.1f, 0.1f, 1.0f }; // Ambient Light (NEW)
static GLfloat g_lightDiffuse[]  =	{ 245.0/255.0, 164.0/255.0, 111.0/255.0, 1.0f }; // Fiffuse Light (NEW)
const GLfloat g_lightPosition1[] = {   .0f,1.0f, .0f, 0.0f }; // Light Source Position (NEW)
const GLfloat g_lightPosition2[] = { -1.0f, .0f, .0f, 0.0f }; // Light Source Position (NEW)
const GLfloat g_lightPosition3[] = {  1.0f, .0f, .0f, 0.0f }; // Light Source Position (NEW)
const GLfloat g_lightPosition4[] = {   .0f,-1.0f, .0f, 0.0f }; // Light Source Position (NEW)
// make a Gl window that draws something:
class MyGlWindow : public Fl_Gl_Window {
public:
	float RX, RY, RZ, TX, TY, TZ, SA;	//for opengl rotate, translate, scale
	int WIDTH, HEIGHT;					//window size
	float old_x, old_y;					//mouse event
	float Inv_Size;						//inverse of model bounding box size
	vector<Vector>	facet_normal;		//array of facet normal for flat-shading

	MyGlWindow(int x, int y, int w, int h) :	WIDTH(w), HEIGHT(h), 
												Fl_Gl_Window(x,y,w,h,"My GL Window"), 
												RX(0), RY(0), RZ(0),
												TX(0), TY(0), TZ(0), SA(1),
												old_x(0), old_y(0)
	{
	};
	void recompute_normal()				// should be called whenever mesh changed!
	{
		facet_normal.clear();			// always remember clear the old dara.
		for(Facet_iterator fi = P.facets_begin(); fi != P.facets_end(); fi++)
		{
			Halfedge_handle he = fi->halfedge();
			Vector norm = CGAL::cross_product( 
						he->next()->vertex()->point() - he->vertex()->point(),
						he->next()->next()->vertex()->point() - he->next()->vertex()->point());
			//normalize the vector.
			double len = norm[0]*norm[0] + norm[1]*norm[1] + norm[2]*norm[2];
			len = sqrt(len);
			norm = norm/len;
			facet_normal.push_back(norm);
		}
	}
	void subdivide_sqrt3()
	{
		sqrt3(P);
		recompute_normal();
	}
	void subdivide_loops()
	{
		//call your loops subdivision function
		loops(P);
		recompute_normal();
	}

	void load_Mesh()
	{
		  if(P.size_of_vertices() != 0)		return;
		  OBJ_Reader<HalfedgeDS> reader;	//my Alias Wavefron .obj reader
		  P.delegate( reader );				// done.

		  ///////////////////////////////////////////////
		  //now shift the entire mesh to the center
		  Vector v = Vector(0,0,0);
		  for(VCI vci = P.vertices_begin(); vci != P.vertices_end(); vci++)
			  v = v + (vci->point() - CGAL::ORIGIN);
		  v = v/P.size_of_vertices();
		  for(Vertex_iterator vi = P.vertices_begin(); vi != P.vertices_end(); vi++)
			  vi->point() = vi->point() - v;

		  ///////////////////////////////////////////////
		  //calculate the bounding box
		  float min_x, min_y, min_z, max_x, max_y, max_z;
		  min_x = min_y = min_z = max_x = max_y = max_z = 0.0f;
		  for(VCI vci = P.vertices_begin(); vci != P.vertices_end(); vci++)
		  {
			  if(vci->point()[0] < min_x)	min_x = vci->point()[0];
			  if(vci->point()[1] < min_y)	min_y = vci->point()[1];
			  if(vci->point()[2] < min_z)	min_z = vci->point()[2];
			  if(vci->point()[0] > max_x)	max_x = vci->point()[0];
			  if(vci->point()[1] > max_y)	max_y = vci->point()[1];
			  if(vci->point()[2] > max_z)	max_z = vci->point()[2];
		  }
		  float x_diag = max_x - min_x, y_diag = max_y - min_y, z_diag = max_z - min_z;
		  float max_diag = (x_diag > y_diag)?x_diag:y_diag;
		  max_diag = (max_diag > z_diag)?max_diag:z_diag;
		  
		  ///////////////////////////////////////////////
		  //get the scaling coeff.
		  Inv_Size = 1.0/max_diag;

		  recompute_normal();
	}
	void save_Mesh()
	{
		if(P.size_of_vertices() == 0)		return;
		Write_OBJ(P);
	}

	int handle(int event) 
	  {
		  switch(event) 
		  {
		  case FL_PUSH:
			  {
				  old_x = Fl::event_x();
				  old_y = Fl::event_y();
			  }
			 return 1;
		  case FL_DRAG:
			  {
				  if(Fl::event_button() == 1)	//left
				  {
					RX += (old_y - Fl::event_y() )*0.3;
					RY += (old_x - Fl::event_x() )*0.3;
				  }
				  else
				  {
					  SA -= (old_y - Fl::event_y() )*0.005;
					  if(SA <= 0.1)	SA = 0.1;
				  }
				  
				  old_x = Fl::event_x();  old_y = Fl::event_y();
			  }
			  return 1;
		  default:
			  return Fl_Gl_Window::handle(event);
		  }
	  }

private:
	Polyhedron P;

	void init_Light()
	{
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
        glLightfv(GL_LIGHT1, GL_AMBIENT , g_lightAmbient);  // Setup The Ambient Light (NEW)
		glLightfv(GL_LIGHT1, GL_DIFFUSE , g_lightDiffuse);  // Setup The Diffuse Light (NEW)
		glLightfv(GL_LIGHT1, GL_POSITION, g_lightPosition1); // Position The Light (NEW)
		glEnable( GL_LIGHT1);    
		glLightfv(GL_LIGHT2, GL_AMBIENT , g_lightAmbient);  // Setup The Ambient Light (NEW)
		glLightfv(GL_LIGHT2, GL_DIFFUSE , g_lightDiffuse);  // Setup The Diffuse Light (NEW)
		glLightfv(GL_LIGHT2, GL_POSITION, g_lightPosition2); // Position The Light (NEW)
		glEnable( GL_LIGHT2);	
		glLightfv(GL_LIGHT3, GL_AMBIENT , g_lightAmbient);  // Setup The Ambient Light (NEW)
		glLightfv(GL_LIGHT3, GL_DIFFUSE , g_lightDiffuse);  // Setup The Diffuse Light (NEW)
		glLightfv(GL_LIGHT3, GL_POSITION, g_lightPosition3); // Position The Light (NEW)
		glEnable( GL_LIGHT3);
		glLightfv(GL_LIGHT4, GL_AMBIENT , g_lightAmbient);  // Setup The Ambient Light (NEW)
		glLightfv(GL_LIGHT4, GL_DIFFUSE , g_lightDiffuse);  // Setup The Diffuse Light (NEW)
		glLightfv(GL_LIGHT4, GL_POSITION, g_lightPosition4); // Position The Light (NEW)
		glEnable( GL_LIGHT4);
	}
	void draw() 
	{
		//printf("shadeMode == %d\n", shadeMode);
		glClearDepth(1.0f);									// Depth Buffer Setup

		glEnable(GL_DEPTH_TEST);
		glDepthMask(GL_TRUE);
		glEnable(GL_CULL_FACE);
		glCullFace(GL_FRONT);			//I don;t know why
		glEnable(GL_NORMALIZE);

		gluPerspective(45.0f,(GLfloat)WIDTH/(GLfloat)HEIGHT,-10.0f,-2000.0f);
		glViewport(0,0,WIDTH, HEIGHT);
		glClearColor(1,1,1,0);	

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer
		glEnable(GL_CULL_FACE);
		glEnable(GL_POLYGON_SMOOTH);
		//glEnable(GL_LINE_SMOOTH);
		glMatrixMode(GL_MODELVIEW);      // Select The Modelview Matrix
		init_Light();
		glLoadIdentity();									// Reset The Current Modelview Matrix
		glPushMatrix();
			glTranslatef(TX, TY, .0f);
			glRotatef(RX, 1.0f, 0.0f, 0.0f); glRotatef(RY, 0.0f, 1.0f, 0.0f); glRotatef(RZ, 0.0f, 0.0f, 1.0f);
			glScalef(Inv_Size*SA, Inv_Size*SA, Inv_Size*SA);
			int i = 0;
			for(Facet_iterator fi = P.facets_begin(); fi != P.facets_end(); fi++, i++)
			{
				Halfedge_handle he = fi->halfedge();
				
				if(SHADING_STATE == 1 || SHADING_STATE == 4)
				{
					glColor3f(0,0,0);
					glBegin(GL_LINE_STRIP);
					{
						for(int i = 0; i < 4; i++)
						{
							glVertex3f(	he->vertex()->point()[0], 
										he->vertex()->point()[1], 
										he->vertex()->point()[2]);
							he = he->next();
						}
					}
					glEnd();
				}
				if(SHADING_STATE == 2 || SHADING_STATE == 4)
				{
					glEnable(GL_LIGHTING);
					glBegin(GL_POLYGON);
					{
						glNormal3f(facet_normal[i][0], facet_normal[i][1], facet_normal[i][2]);
						for(int i = 0; i < 3; i++)
						{
							glVertex3f(	he->vertex()->point()[0], 
										he->vertex()->point()[1], 
										he->vertex()->point()[2]);
							he = he->next();
						}
					}
					glEnd();
					glDisable(GL_LIGHTING);			
				}
			}
		glPopMatrix();
	};
};